from concurrent.futures import Future, wait
import logging
import sys
import tempfile

from iopipe.compat import StringIO
from iopipe.plugins import Plugin
from iopipe.signer import get_signed_request

from .formatter import JSONFormatter
from .request import upload_log_data
from .stream import StreamToLogger
from .wrapper import LogWrapper


class LoggerPlugin(Plugin):
    name = "logger"
    version = "0.1.0"
    homepage = "https://github.com/iopipe/iopipe-python#logger-plugin"
    enabled = True

    def __init__(
        self, name=None, level=logging.INFO, redirect_stdout=True, use_tmp=False
    ):
        """
        Instantiates the logger plugin

        :param name: Specify custom log name.
        :type name: str
        :param level: Specify a log level for the handler.
        :type level: int
        :param redirect_stdout: Whether or not to redirect stdout.
        :type redirect_print: bool
        :param use_tmp: Write logs to the /tmp directory instead of a memory buffer.
        :type use_tmp: bool
        """
        formatter = JSONFormatter()

        self.handler = logging.StreamHandler(StringIO())
        self.handler.setFormatter(formatter)
        self.handler.setLevel(level)

        self.logger = logging.getLogger(name)
        self.logger.addHandler(self.handler)
        self.logger.setLevel(level)

        self.redirect_stdout = redirect_stdout
        self.use_tmp = use_tmp

    def pre_setup(self, iopipe):
        self.iopipe = iopipe

    def post_setup(self, iopipe):
        if iopipe.config["debug"] is True:
            self.handler.setLevel(logging.DEBUG)
            self.logger.setLevel(logging.DEBUG)

    def pre_invoke(self, event, context):
        self.context = context
        self.context.iopipe.register(
            "log", LogWrapper(self.logger, context), force=True
        )

        if self.use_tmp is True:
            self.handler.stream = tempfile.NamedTemporaryFile(delete=False, mode="w")
        else:
            self.handler.stream = StringIO()

        if self.redirect_stdout is True:
            sys.stdout = StreamToLogger(self.logger)

    def post_invoke(self, event, context):
        self.signed_request = None
        self.handler.flush()
        if self.handler.stream.tell():
            self.signed_request = self.iopipe.submit_future(
                get_signed_request, self.iopipe.config["token"], self.context, ".log"
            )
        if self.redirect_stdout is True:
            sys.stdout = sys.__stdout__

    def pre_report(self, report):
        if self.handler.stream.tell():
            stream = self.handler.stream
            if hasattr(stream, "getvalue"):
                stream = StringIO(stream.getvalue())
            if hasattr(stream, "file"):
                stream = stream.name
                self.handler.stream.close()
            if self.signed_request is not None:
                if isinstance(self.signed_request, Future):
                    wait([self.signed_request])
                self.signed_request = self.signed_request.result()
            if (
                self.signed_request is not None
                and "signedRequest" in self.signed_request
            ):
                self.iopipe.submit_future(
                    upload_log_data, self.signed_request["signedRequest"], stream
                )
                if "jwtAccess" in self.signed_request:
                    plugin = next((p for p in report.plugins if p["name"] == self.name))
                    if "uploads" not in plugin:
                        plugin["uploads"] = []
                    plugin["uploads"].append(self.signed_request["jwtAccess"])

    def post_report(self, report):
        pass



import decimal
import numbers
import warnings
from . import constants
from .compat import string_types


class ContextWrapper(object):
    def __init__(self, base_context, instance):
        self.base_context = base_context
        self.instance = instance
        self.iopipe = IOpipeContext(self.instance)

    def __getattr__(self, name):
        return getattr(self.base_context, name)


class IOpipeContext(object):
    def __init__(self, instance):
        self.instance = instance

    def metric(self, key, value):
        if self.instance.report is None:
            warnings.warn(
                "Attempting to add metrics before function decorated with IOpipe. "
                "This metric will not be recorded."
            )
            return

        name = str(key)
        if len(name) > constants.METRIC_NAME_LIMIT:
            warnings.warn(
                "Custom metric of name %s is longer than allowed limit of "
                "%s characters. "
                "This metric will not be recorded."
                % (name, constants.METRIC_NAME_LIMIT)
            )
            return

        event = {"name": name}

        # Add numerical values to report
        # We typecast decimals as strings: not JSON serializable and casting to floats
        # can result in rounding errors.
        if isinstance(value, numbers.Number) and not isinstance(value, decimal.Decimal):
            event["n"] = value
        else:
            event["s"] = str(value)

        self.instance.report.custom_metrics.append(event)

        if not name.startswith("@iopipe"):
            self.label("@iopipe/metrics")

    log = metric

    def label(self, name):
        if not isinstance(name, string_types):
            warnings.warn(
                "Attempted to add a label that is not of type string. "
                "This label will not be recorded."
            )
            return

        if len(name) > constants.METRIC_NAME_LIMIT:
            warnings.warn(
                "Label of name %s is longer than allowed limit of "
                "%s characters. "
                "This label will not be recorded." % (name, constants.METRIC_NAME_LIMIT)
            )
            return

        self.instance.report.labels.add(name)

    def error(self, error):
        if self.instance.report is None:
            warnings.warn(
                "An exception occurred before function was decorated with IOpipe. "
                "This exception will not be recorded."
            )
            raise error

        self.instance.report.prepare(error)
        self.instance.run_hooks("pre:report")
        self.instance.report.send()
        self.instance.run_hooks("post:report")
        raise error

    def register(self, name, value, force=False):
        if not hasattr(self, name) or force:
            setattr(self, name, value)

    def unregister(self, name):
        if hasattr(self, name):
            delattr(self, name)



import collections
import time

from iopipe.monotonic import monotonic

Entry = collections.namedtuple(
    "Entry", ["name", "startTime", "duration", "entryType", "timestamp"]
)


def time_in_millis(time=None, offset=0):
    if time is None:
        time = monotonic()
    return time * 1000 - offset


def get_offset(timeline):
    return timeline.offset or timeline.init_time


def mark_data(
    timeline, name, start_time=None, duration=0, entry_type="mark", timestamp=None
):
    data = timeline.data or []
    entry = Entry(
        name=name,
        startTime=start_time or time_in_millis(offset=get_offset(timeline)),
        duration=duration,
        entryType=entry_type,
        timestamp=timestamp or int(time.time() * 1000),
    )
    data.append(entry)
    data.sort(key=lambda i: i.startTime)
    return data, entry


class Timeline(object):
    def __init__(self, offset=0):
        self.init_time = time_in_millis()
        self.offset = offset
        self.data = []

    def mark(self, name):
        self.data, entry = mark_data(self, name=name)
        return entry

    def get_entries(self):
        return self.data

    def get_entries_by_name(self, name):
        return [d for d in self.data if d.name == name]

    def get_entries_by_type(self, type):
        return [d for d in self.data if d.entryType == type]

    def measure(self, name, start, end=None):
        start_mark = self.get_entries_by_name(start)
        start_mark = start_mark[-1] if start_mark else None
        start_time = (
            start_mark.startTime if start_mark else self.init_time - get_offset(self)
        )
        timestamp = start_mark.timestamp if start_mark else None

        end_time = self.now()
        if end is not None:
            end_mark = self.get_entries_by_name(end)
            end_mark = end_mark[-1] if end_mark else None
            end_time = end_mark.startTime if end_mark else end_time

        duration = end_time - start_time

        self.data, entry = mark_data(
            self,
            name=name,
            start_time=start_time,
            duration=duration,
            entry_type="measure",
            timestamp=timestamp,
        )
        return entry

    def clear_marks(self):
        self.data = [d for d in self.data if d["entryType"] != "mark"]

    def clear_measures(self):
        self.data = [d for d in self.data if d["entryType"] != "measure"]

    def clear(self):
        self.data = []

    def delete(self, name):
        self.data = [d for d in self.data if name not in d.name]

    def now(self):
        return time_in_millis(offset=get_offset(self))



from distutils.util import strtobool
import os

from iopipe.plugins import Plugin

from .auto_http import patch_requests, restore_requests
from .marker import Marker
from .timeline import Timeline
from .util import add_timeline_measures


class TracePlugin(Plugin):
    name = "trace"
    version = "1.1.0"
    homepage = "https://github.com/iopipe/iopipe-python#trace-plugin"
    enabled = True

    def __init__(self, auto_measure=True, auto_http=False, http_filter=None):
        """
        Instantiates the trace plugin

        :param auto_measure: Whether or not to automatically measure traces
        :type auto_measure: bool
        :param auto_http: Whether or not to automatically trace HTTP requests
        :type auto_http: bool
        :param http_filter: A callable to filter http requests
        :type http_filter: function
        """
        self.auto_measure = auto_measure
        self.auto_http = auto_http is True or strtobool(
            os.getenv("IOPIPE_TRACE_AUTO_HTTP_ENABLED", "false")
        )
        self.http_filter = http_filter

        self.timeline = Timeline()

    def pre_setup(self, iopipe):
        pass

    def post_setup(self, iopipe):
        pass

    def pre_invoke(self, event, context):
        self.timeline = Timeline()
        context.iopipe.register("mark", Marker(self.timeline, context))

        if self.auto_http is True:
            patch_requests(context, self.http_filter)

    def post_invoke(self, event, context):
        context.iopipe.unregister("mark")

        if self.auto_http is True:
            restore_requests()

    def pre_report(self, report):
        if self.auto_measure:
            add_timeline_measures(self.timeline)

        for entry in self.timeline.get_entries():
            report.performance_entries.append(entry._asdict())

    def post_report(self, report):
        pass



import logging
import os

try:
    import requests
except ImportError:
    from botocore.vendored import requests

logger = logging.getLogger(__name__)


def upload_profiler_report(url, filename):
    """
    Uploads a profiler report to IOpipe

    :param url: The signed URL
    :param filename: The profiler report file.
    """
    try:
        logger.debug("Uploading profiler report to IOpipe")
        with open(filename, "rb") as data:
            response = requests.put(url, data=data)
        response.raise_for_status()
    except Exception as e:
        logger.debug("Error while uploading profiler report: %s", e)
        if hasattr(e, "response"):
            logger.debug(e.response.content)
    else:
        logger.debug("Profiler report uploaded successfully")
    finally:
        if os.path.isfile(filename):
            os.remove(filename)



from iopipe.plugins import Plugin

from .event_types import metrics_for_event_type


class EventInfoPlugin(Plugin):
    name = "event-info"
    version = "1.2.0"
    homepage = "https://github.com/iopipe/iopipe-python#event-info-plugin"
    enabled = True

    def pre_setup(self, iopipe):
        pass

    def post_setup(self, iopipe):
        pass

    def pre_invoke(self, event, context):
        pass

    def post_invoke(self, event, context):
        metrics_for_event_type(event, context)

    def pre_report(self, report):
        pass

    def post_report(self, report):
        pass



import logging
import os

try:
    import requests
except ImportError:
    from botocore.vendored import requests

from iopipe.compat import StringIO

logger = logging.getLogger(__name__)


def upload_log_data(url, stream_or_file):
    """
    Uploads log data to IOpipe.

    :param url: The signed URL
    :param stream_or_file: The log data stream or file
    """
    try:
        logger.debug("Uploading log data to IOpipe")
        if isinstance(stream_or_file, StringIO):
            stream_or_file.seek(0)
            response = requests.put(url, data=stream_or_file, timeout=None)
        else:
            with open(stream_or_file, "rb") as data:
                response = requests.put(url, data=data, timeout=None)
        response.raise_for_status()
    except Exception as e:
        logger.debug("Error while uploading log data: %s", e)
        logger.exception(e)
        if hasattr(e, "response") and hasattr(e.response, "content"):
            logger.debug(e.response.content)
    else:
        logger.debug("Log data uploaded successfully")
    finally:
        if isinstance(stream_or_file, str) and os.path.exists(stream_or_file):
            os.remove(stream_or_file)



import io
import sys

PY3 = sys.version_info[0] == 3

if PY3:
    from urllib.parse import urlparse

    string_types = (str,)
    StringIO = io.StringIO
else:
    from urlparse import urlparse  # noqa

    string_types = (basestring,)  # noqa
    StringIO = io.BytesIO



import collections
import uuid

try:
    from requests.sessions import Session
except ImportError:
    Session = None

try:
    from botocore.vendored.requests.sessions import Session as BotocoreSession
except ImportError:
    BotocoreSession = None

from iopipe.compat import urlparse

if Session is not None:
    original_session_send = Session.send

if BotocoreSession is not None:
    original_botocore_session_send = BotocoreSession.send

INCLUDE_HEADERS = [
    "accept",
    "accept-encoding",
    "age",
    "cache-control",
    "connection",
    "content-encoding",
    "content-length",
    "content-type",
    "date",
    "etag",
    "host",
    "server",
    "strict-transport-security",
    "user-agent",
    "vary",
]

Request = collections.namedtuple(
    "Request",
    [
        "hash",
        "headers",
        "hostname",
        "method",
        "path",
        "pathname",
        "port",
        "protocol",
        "query",
        "url",
    ],
)

Response = collections.namedtuple(
    "Response", ["headers", "statusCode", "statusMessage"]
)


def patch_session_send(context, http_filter):
    """
    Monkey patches requests' Session class, if available. Overloads the
    send method to add tracing and metrics collection.
    """
    if Session is None:
        return

    def send(self, *args, **kwargs):
        id = str(uuid.uuid4())
        with context.iopipe.mark(id):
            response = original_session_send(self, *args, **kwargs)
        trace = context.iopipe.mark.measure(id)
        context.iopipe.mark.delete(id)
        collect_metrics_for_response(response, context, trace, http_filter)
        return response

    Session.send = send


def patch_botocore_session_send(context, http_filter):
    """
    Monkey patches botocore's vendored requests, if available. Overloads the
    Session class' send method to add tracing and metric collection.
    """
    if BotocoreSession is None:
        return

    def send(self, *args, **kwargs):
        id = str(uuid.uuid4())
        with context.iopipe.mark(id):
            response = original_botocore_session_send(self, *args, **kwargs)
        trace = context.iopipe.mark.measure(id)
        context.iopipe.mark.delete(id)
        collect_metrics_for_response(response, context, trace, http_filter)
        return response

    BotocoreSession.send = send


def restore_session_send():
    """Restores the original Session send method"""
    if Session is not None:
        Session.send = original_session_send


def restore_botocore_session_send():
    """Restores the original botocore Session send method"""
    if BotocoreSession is not None:
        BotocoreSession.send = original_botocore_session_send


def patch_requests(context, http_filter):
    patch_session_send(context, http_filter)
    patch_botocore_session_send(context, http_filter)


def restore_requests():
    restore_session_send()
    restore_botocore_session_send()


def collect_metrics_for_response(http_response, context, trace, http_filter):
    """
    Collects relevant metrics from a requests Response object and adds them to
    the IOpipe context.
    """
    if http_filter is not None and callable(http_filter):
        http_response = http_filter(http_response)
        if http_response is False:
            return

    request = None
    if hasattr(http_response, "request"):
        parsed_url = None
        if hasattr(http_response.request, "url"):
            parsed_url = urlparse(http_response.request.url)

        request_headers = []
        if hasattr(http_response.request, "headers"):
            request_headers = [
                {"key": k, "string": v}
                for k, v in http_response.request.headers.items()
                if k.lower() in INCLUDE_HEADERS
            ]

        request = Request(
            hash=getattr(parsed_url, "fragment", None),
            headers=request_headers,
            hostname=getattr(parsed_url, "hostname", None),
            method=getattr(http_response.request, "method", None),
            path=getattr(parsed_url, "path", None),
            # TODO: Determine if this is redundant
            pathname=getattr(parsed_url, "path", None),
            port=getattr(parsed_url, "port", None),
            protocol=getattr(parsed_url, "scheme", None),
            query=getattr(parsed_url, "query", None),
            url=getattr(http_response.request, "url", None),
        )

    response_headers = []
    if hasattr(http_response, "headers"):
        response_headers = [
            {"key": k, "string": v}
            for k, v in http_response.headers.items()
            if k.lower() in INCLUDE_HEADERS
        ]

    response = Response(
        headers=response_headers,
        statusCode=getattr(http_response, "status_code", None),
        statusMessage=None,
    )

    context.iopipe.mark.http_trace(trace, request, response)



import concurrent.futures as futures
import functools
import inspect
import logging
import signal
import warnings

from .config import set_config
from .context import ContextWrapper
from .plugins import is_plugin
from .report import Report

logging.basicConfig()

logger = logging.getLogger("iopipe")
logger.setLevel(logging.INFO)


class MockFuture(object):
    def __init__(self, func, *args, **kwargs):
        self._result = func(*args, **kwargs)

    def result(self):
        return self._result


class TimeoutError(Exception):
    pass


class IOpipeCore(object):
    """
    The stock IOpipe agent, without any plugins loaded.
    """

    def __init__(self, token=None, url=None, debug=None, plugins=None, **options):
        self.plugins = []
        if plugins is not None and isinstance(plugins, list):
            self.plugins = self.load_plugins(plugins)
            options["plugins"] = self.plugins

        self.run_hooks("pre:setup")

        if token is not None:
            options["token"] = token
        if url is not None:
            options["url"] = url
        if debug is not None:
            options["debug"] = debug

        self.config = set_config(**options)
        self.config["plugins"] = self.load_plugins(self.config["plugins"])
        self.futures = []
        self.pool = futures.ThreadPoolExecutor()
        self.report = None

        if self.config["debug"]:
            logger.setLevel(logging.DEBUG)

        self.run_hooks("post:setup")

    def __del__(self):
        self.pool.shutdown()

    def log(self, key, value):
        if self.report is None:
            warnings.warn(
                "Attempting to log metrics before function decorated with IOpipe. "
                "This metric will not be recorded."
            )
            return

        self.report.context.iopipe.log(key, value)

    def error(self, error):
        if self.report is None:
            warnings.warn(
                "An exception occurred before function was decorated with IOpipe. "
                "This exception will not be recorded."
            )
            raise error

        self.report.context.iopipe.error(error)

    err = error

    def __call__(self, func):
        @functools.wraps(func)
        def wrapped(event, context):
            logger.debug("%s wrapped with IOpipe decorator" % repr(func))

            context = ContextWrapper(context, self)

            self.run_hooks("pre:invoke", event=event, context=context)

            # if env var IOPIPE_ENABLED is set to False skip reporting
            if self.config["enabled"] is False:
                logger.debug("IOpipe agent disabled, skipping reporting")
                return func(event, context)

            # If a token is not present, skip reporting
            if not self.config["token"]:
                warnings.warn(
                    "Your function is decorated with iopipe, but a valid token was not found. "
                    "Set the IOPIPE_TOKEN environment variable with your IOpipe project token."
                )
                return func(event, context)

            self.report = Report(self, context)

            signal.signal(signal.SIGALRM, self.handle_timeout)

            # Disable timeout if timeout_window <= 0, or if our context doesn't have a get_remaining_time_in_millis
            if (
                self.config["timeout_window"] > 0
                and hasattr(context, "get_remaining_time_in_millis")
                and callable(context.get_remaining_time_in_millis)
            ):
                timeout_duration = (
                    context.get_remaining_time_in_millis() / 1000.0
                ) - self.config["timeout_window"]

                # The timeout_duration cannot be a negative number, disable if it is
                timeout_duration = max([0, timeout_duration])

                # Maximum execution time is 10 minutes, make sure timeout doesn't exceed that minus the timeout window
                timeout_duration = min(
                    [timeout_duration, 60 * 60 * 10 - self.config["timeout_window"]]
                )

                logger.debug("Setting timeout duration to %s" % timeout_duration)

                # Using signal.setitimer instead of signal.alarm because the latter only accepts integers and we want to
                # be able to timeout at millisecond granularity
                signal.setitimer(signal.ITIMER_REAL, timeout_duration)

            result = None

            try:
                result = func(event, context)
            except Exception as e:
                self.run_hooks("post:invoke", event=event, context=context)

                # This prevents this block from being executed a second time in the event that a timeout occurs and an
                # exception is subsequently raised within the handler
                if self.report.sent is False:
                    self.report.prepare(e)
                    self.run_hooks("pre:report")
                    self.report.send()
                    self.run_hooks("post:report")

                raise e
            else:
                self.run_hooks("post:invoke", event=event, context=context)
                self.report.prepare()
                self.run_hooks("pre:report")
                self.report.send()
                self.run_hooks("post:report")
            finally:
                signal.setitimer(signal.ITIMER_REAL, 0)
                self.wait_for_futures()

            return result

        return wrapped

    decorator = __call__

    def handle_timeout(self, signum, frame):
        """
        Catches a timeout (SIGALRM) and sends the report before actual timeout occurs.

        The signum and frame parameters are passed by the signal module to this handler.

        :param signum: The signal number being handled.
        :param frame: The stack frame when signal was raised.
        """
        logger.debug("Function is about to timeout, sending report")
        self.report.prepare(TimeoutError("Timeout Exceeded."), frame)
        self.run_hooks("pre:report")
        self.report.send()
        self.run_hooks("post:report")
        self.wait_for_futures()

    def load_plugins(self, plugins):
        """
        Loads plugins that match the `Plugin` interface and are instantiated.

        :param plugins: A list of plugin instances.
        """

        def instantiate(plugin):
            return plugin() if inspect.isclass(plugin) else plugin

        loaded_plugins = []
        plugins_seen = []

        # Iterate over plugins in reverse to permit users to override default
        # plugin config
        for plugin in reversed(plugins):
            if not is_plugin(plugin) or plugin.name in plugins_seen:
                continue
            # Build the plugins list in reverse to restore original order
            loaded_plugins.insert(0, instantiate(plugin))
            plugins_seen.append(plugin.name)

        return loaded_plugins

    def run_hooks(self, name, event=None, context=None):
        """
        Runs plugin hooks for each registered plugin.
        """
        hooks = {
            "pre:setup": lambda p: p.pre_setup(self),
            "post:setup": lambda p: p.post_setup(self),
            "pre:invoke": lambda p: p.pre_invoke(event, context),
            "post:invoke": lambda p: p.post_invoke(event, context),
            "pre:report": lambda p: p.pre_report(self.report),
            "post:report": lambda p: p.post_report(self.report),
        }

        if name in hooks:
            [hooks[name](p) for p in self.plugins if p.enabled]

    def submit_future(self, func, *args, **kwargs):
        """
        Submit a function call to be run as a future in a thread pool. This
        should be an I/O bound operation.
        """
        # This mode will run futures synchronously. This should only be used
        # for benchmarking purposes.
        if self.config["sync_http"] is True:
            return MockFuture(func, *args, **kwargs)

        future = self.pool.submit(func, *args, **kwargs)
        self.futures.append(future)
        return future

    def wait_for_futures(self):
        """
        Wait for all futures to complete. This should be done at the end of an
        invocation.
        """
        [future for future in futures.as_completed(self.futures)]
        self.futures = []



from __future__ import division

import random
import uuid

from .system import read_hostname  # noqa


def read_bootid():
    """
    Mocks read_bootid as this is a Linux-specific operation.
    """
    return uuid.uuid4().hex


def read_disk():
    """
    Mocks read_disk as this is a Linux-specific operation.
    """
    used = random.randint(0, 500)
    return {
        "totalMiB": 500,
        "usedMiB": used,
        "usedPercentage": round((used / 500) * 100, 2),
    }


def read_meminfo():
    """
    Mocks read_meminfo as this is a Linux-specific operation.
    """
    return {
        "MemTotal": random.randint(0, 999999999),
        "MemFree": random.randint(0, 999999999),
        "MemAvailable": random.randint(0, 999999999),
        "HugePages_Total": random.randint(0, 999999999),
    }


def read_pid_stat(pid):
    """
    Mocks read_pid_stat as this is a Linux-specific operation.
    """
    return {
        "utime": random.randint(0, 999999999),
        "stime": random.randint(0, 999999999),
        "cutime": random.randint(0, 999999999),
        "cstime": random.randint(0, 999999999),
    }


def read_pid_status(pid):
    """
    Mocks read_pid_status as this is a Linux-specific operation.
    """
    return {
        "FDSize": random.randint(0, 999999999),
        "Threads": random.randint(0, 99999),
        "VmRSS": random.randint(0, 999999999),
    }


def read_stat():
    """
    Mocks read_stat as this is a Linux-specific operation.
    """
    return [
        {
            "times": {
                "user": random.randint(0, 999999999),
                "nice": random.randint(0, 999999999),
                "sys": random.randint(0, 999999999),
                "idle": random.randint(0, 999999999),
                "irq": random.randint(0, 999999999),
            }
        }
    ]


def read_uptime():
    """
    Mocks read_updtime as this is a Linux-specific operaiton.
    """
    return random.randint(0, 999999999)



def add_timeline_measures(timeline):
    entries = timeline.get_entries_by_type("mark")
    measures = timeline.get_entries_by_type("measure")
    entry_names = [
        e.name
        for e in entries
        if e.name.startswith("start:") or e.name.startswith("end:")
    ]
    measure_names = [m.name for m in measures]
    for name in entry_names:
        if name.startswith("start:"):
            _, base_name = name.split(":", 1)
            end_name = "end:%s" % base_name
            measure_name = "measure:%s" % base_name
            if end_name in entry_names and measure_name not in measure_names:
                timeline.measure(measure_name, name, end_name)



import logging

try:
    import requests
except ImportError:
    from botocore.vendored import requests

logger = logging.getLogger(__name__)
session = requests.Session()


def send_report(report, config):
    """
    Sends the report to IOpipe's collector.

    :param report: The report to be sent.
    :param config: The IOpipe agent configuration.
    """
    url = "https://{host}{path}".format(**config)

    try:
        response = session.post(url, json=report, timeout=config["network_timeout"])
        response.raise_for_status()
        logger.debug("Report sent to IOpipe successfully")
    except Exception as e:
        logger.debug("Error sending report to IOpipe: %s" % e)



import os
import sys

from setuptools import find_packages, setup

try:
    from pypandoc import convert

    README = convert("README.md", "rst")
except (ImportError, OSError):
    README = open(os.path.join(os.path.dirname(__file__), "README.md"), "r").read()

install_requires = []
if sys.version_info[0] == 2:
    install_requires.append("futures")

setup(
    name="iopipe",
    version="1.7.0",
    description="IOpipe agent for serverless Application Performance Monitoring",
    long_description=README,
    author="IOpipe",
    author_email="dev@iopipe.com",
    url="https://github.com/iopipe/iopipe-python",
    packages=find_packages(exclude=("tests", "tests.*")),
    extras_require={
        "dev": ["black", "jmespath>=0.7.1,<1.0.0", "pre-commit", "requests"]
    },
    install_requires=install_requires,
    setup_requires=["pytest-runner"],
    tests_require=[
        "coverage",
        "jmespath>=0.7.1,<1.0.0",
        "mock",
        "pytest",
        "pytest-benchmark",
        "pytest-catchlog",
        "pytest-cov",
        "requests",
    ],
    zip_safe=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
)
